@@ -1,4 +1,4 @@
-//  $Id: RtConfig.cc,v 4.11.2.3 2003/10/21 09:08:32 katie Exp $
+//  $Id: RtConfig.cc,v 4.11.2.4 2004/07/30 10:58:10 katie Exp $
 // Copyright (c) 2001,2002                        RIPE NCC
 //
 // All Rights Reserved
@@ -59,8 +59,8 @@ extern "C" {
 #   include <unistd.h>
 #endif
 }
-#include <iostream.h>
-#include <iomanip.h>
+#include <iostream>
+#include <iomanip>
 #include "irr/irr.hh"
 #include "irr/rawhoisc.hh"
 #include "irr/ripewhoisc.hh"
@@ -79,6 +79,8 @@ extern "C" {
 #include "rpsl/schema.hh"
 #include "irr/classes.hh"
 
+using namespace std;
+
 bool opt_rusage                  = false;
 Rusage ru(clog, &opt_rusage);
 
