@@ -1,4 +1,4 @@
-//  $Id: irr.cc,v 4.11.2.2 2003/10/17 08:38:44 katie Exp $
+//  $Id: irr.cc,v 4.11.2.3 2004/07/30 10:58:11 katie Exp $
 // Copyright (c) 2001,2002                        RIPE NCC
 //
 // All Rights Reserved
@@ -53,7 +53,8 @@
 
 #include "config.h"
 #include <cstdio>
-#include <fstream.h>
+#include <fstream>
+#include <iostream>
 #include "gnu/prefixranges.hh"
 #include "gnug++/SetOfUInt.hh"
 #include "irr.hh"
@@ -62,6 +63,8 @@
 #include "cache.hh"
 #include "classes.hh"
 
+using namespace std;
+
 IRR *irr;
 ProtocolName protocolName[] = {
   { IRR::rawhoisd, "rawhoisd" },
