@@ -1,4 +1,4 @@
-//  $Id: NE.cc,v 4.9.4.4 2003/10/17 08:46:55 katie Exp $
+//  $Id: NE.cc,v 4.9.4.6 2004/07/30 10:58:12 katie Exp $
 // Copyright (c) 2001,2002                        RIPE NCC
 //
 // All Rights Reserved
@@ -53,7 +53,7 @@
 //             Katie Petrusha <katie@ripe.net>
 
 #include "config.h"
-#include <iostream.h>
+#include <iostream>
 #include "NE.hh"
 #include "util/debug.hh"
 #include "irr/irr.hh"
@@ -62,6 +62,8 @@
 #include "gnug++/SetOfUInt.hh"
 #include "gnu/prefixranges.hh"
 
+using namespace std;
+
 #define DBG_REDUCE 3
 #define DBG_REDUCE_DETAIL 4
 #define DBG_OR 5
@@ -597,19 +599,19 @@ void NormalExpression::restrict(FilterAFI *af) {
      return;
    }
 
-   for (Item *afi_item = af->afi_list->head(); afi_item; afi_item = af->afi_list->next(afi_item)) {
-
-     if (((ItemAFI *) afi_item)->is_ipv6()) { //v6
-       this->singleton_flag = NormalTerm::IPV6_PRFX;
-       for (term = ne->first(); term ; term = ne->next())
-         if (term->prfx_set.universal())
-           *this += term;
-     } 
-     else { //v4
-       this->singleton_flag = NormalTerm::PRFX;
-       for (term = ne->first(); term ; term = ne->next())
-         if (term->ipv6_prfx_set.universal())
-           *this += term;
+   for (term = ne->first(); term ; term = ne->next()) {
+     if (term->prfx_set.universal()) { // v6
+       term->ipv6_prfx_set.restrict(af->afi_list);   
+       if (! term->ipv6_prfx_set.isEmpty()) {
+         *this += term;
+         this->singleton_flag = NormalTerm::IPV6_PRFX;
+       }
+     } else if (term->ipv6_prfx_set.universal()) {
+       term->prfx_set.restrict(af->afi_list);   
+       if (! term->prfx_set.isEmpty()) {
+         *this += term;
+         this->singleton_flag = NormalTerm::PRFX;
+       }
      }
    }
 }
