@@ -1,4 +1,4 @@
-//  $Id: SetOfPrefix.cc,v 4.9.4.1 2003/06/27 12:49:46 katie Exp $
+//  $Id: SetOfPrefix.cc,v 4.9.4.2 2004/07/30 10:58:13 katie Exp $
 // Copyright (c) 2001,2002                        RIPE NCC
 //
 // All Rights Reserved
@@ -59,6 +59,9 @@
 #include <cstdio>
 #include <cassert>
 #include "rpsl/prefix.hh"
+#include <ostream>
+
+using namespace std;
 
 void SetOfPrefix::insert(const PrefixRanges& b) { 
    if (_universal)
