@@ -1,4 +1,4 @@
-/* $Id: re2dfa.c,v 4.9 2002/04/09 11:48:47 katie Exp $
+/* $Id: re2dfa.c,v 4.9.4.1 2004/07/30 10:58:13 katie Exp $
 // Copyright (c) 2001,2002                        RIPE NCC
 //
 // All Rights Reserved
@@ -78,8 +78,9 @@ struct _rd_bit {
 
 void debugging_hack() {
    /* this function lets me put a break point here... */
-   fprintf(stderr, "InternalError: Too many states in AS path expression
-InternalError: Increase RD_MAXASPSTATES in src/re2dfa/re2dfa.h and rebuild.\n");
+   fprintf(stderr,
+           "InternalError: Too many states in AS path expression\n"
+           "InternalError: Increase RD_MAXASPSTATES in src/re2dfa/re2dfa.h and rebuild.\n");
 }
 
 #define RD_ASSIGN_BIT(s) {					\
