@@ -1,4 +1,4 @@
-//  $Id: prefix.cc,v 4.9.4.11 2003/10/01 09:48:00 katie Exp $
+//  $Id: prefix.cc,v 4.9.4.14 2004/07/30 10:58:14 katie Exp $
 // Copyright (c) 2001,2002                        RIPE NCC
 //
 // All Rights Reserved
@@ -55,13 +55,15 @@
 #include "config.h"
 #include <cstdio>
 #include <cstring>
-#include <iostream.h>
+#include <iostream>
 #include "prefix.hh"
 #include <stdlib.h>
 #include <sys/types.h>
 #include <algorithm>
 #include <iterator>
 
+using namespace std;
+
 static unsigned int masks[] ={ 0x00000000,
 		 0x80000000, 0xC0000000, 0xE0000000, 0xF0000000,
 		 0xF8000000, 0xFC000000, 0xFE000000, 0xFF000000,
@@ -86,6 +88,7 @@ static unsigned long long int bits[] = { 0x100000000ULL,
 
 static ip_v6word_t ipv6bits[] = { 
 0x0000000000000000ULL, // extra bit is used to cover numbers > 64 bit
+
 0x8000000000000000ULL,0x4000000000000000ULL,0x2000000000000000ULL, 0x1000000000000000ULL,
 0x0800000000000000ULL,0x0400000000000000ULL,0x0200000000000000ULL, 0x0100000000000000ULL,
 0x0080000000000000ULL,0x0040000000000000ULL,0x0020000000000000ULL, 0x0010000000000000ULL,
@@ -142,7 +145,7 @@ unsigned int quad2int(char *quad) {
    return i;
 }
 
-ostream& operator<<(ostream& stream, const PrefixRange& p) {
+std::ostream& operator<<(std::ostream& stream, const PrefixRange& p) {
    stream << p.get_text();
    return stream;
 }
@@ -426,11 +429,13 @@ IPAddr::IPAddr(char *name) {
 PrefixRange NullPrefixRange("0.0.0.0/32^32-32");
 Prefix      NullPrefix("0.0.0.0/32");
 IPAddr      NullIPAddr("0.0.0.0");
+PrefixRange MulticastPrefixRange("224.0.0.0/4^+");
 
 IPv6PrefixRange NullIPv6PrefixRange("::/128^128-128");
 IPv6Prefix      NullIPv6Prefix("::/128");
 IPv6Addr        NullIPv6Addr("::");
 ipv6_addr_t     NullIPv6(0,0);
+IPv6PrefixRange MulticastIPv6PrefixRange("ff00::/8^8-128");
 
 /* IPv6 stuff */
 
@@ -450,8 +455,19 @@ char* ipv62hex(ipv6_addr_t *ip, char *buffer){
    i[5] = (low >> 32) & 0xFFFF;
    i[6] = (low >> 16) & 0xFFFF;
    i[7] = low & 0xFFFF;
-   
+
    memset(buffer, 0, strlen(buffer));
+ 
+
+   // do non-compressed
+
+  /* while (j <= 7) {
+     sprintf(buffer + strlen(buffer), "%X", i[j]); 
+     if (j != 7)
+       sprintf(buffer + strlen(buffer), ":");
+     j++;
+   }
+  */
 
    while ((j <= 7) && (i[j] != 0)) {
      sprintf(buffer + strlen(buffer), "%X", i[j]); 
@@ -469,7 +485,7 @@ char* ipv62hex(ipv6_addr_t *ip, char *buffer){
      sprintf(buffer + strlen(buffer), ":");
    }
 
-   while ((j <= 7) && (i[j] != 0)) {
+   while (j <= 7) {
      sprintf(buffer + strlen(buffer), "%X", i[j]);
      if (j != 7)
       sprintf(buffer + strlen(buffer), ":");
@@ -963,9 +979,14 @@ char *MPPrefix::get_ip_text() const {
 }
 
 char *MPPrefix::get_afi() const {
-  if (ipv4)
-     return strdup("ipv4");
-  return strdup("ipv6");
+  if (ipv4) {
+     if (MulticastPrefixRange.contains(*ipv4)) 
+       return strdup("ipv4.multicast");
+     return strdup("ipv4.unicast");
+  }
+  if (MulticastIPv6PrefixRange.contains(*ipv6))
+       return strdup("ipv6.multicast");
+  return strdup("ipv6.unicast");
 }
 
 bool MPPrefix::makeMoreSpecific(int code, int n, int m) {
@@ -1021,7 +1042,7 @@ ipv6_addr_t& operator|(ipv6_addr_t one, ipv6_addr_t two)
 
 }
 
-ipv6_addr_t& ipv6_addr_t::operator|(unsigned int i)
+ipv6_addr_t& ipv6_addr_t::operator|(ip_v6word_t i)
 {
   ipv6_addr_t *t = new ipv6_addr_t(0,0,0);
 
@@ -1032,13 +1053,45 @@ ipv6_addr_t& ipv6_addr_t::operator|(unsigned int i)
   return *t;
 }
 
-ipv6_addr_t& ipv6_addr_t::operator<<(unsigned int i)
+ipv6_addr_t& ipv6_addr_t::operator+(ip_v6word_t i)
+{  
+  ipv6_addr_t *t = new ipv6_addr_t(0,0,0);
+
+  t->xbit = xbit;
+  t->high = high;
+  t->low  = low + i;
+
+  return *t;
+}
+
+ipv6_addr_t& ipv6_addr_t::operator=(ip_v6word_t i)
 {
   ipv6_addr_t *t = new ipv6_addr_t(0,0,0);
 
-  t->xbit = (xbit << i) | (high >> (64-i)); 
-  t->high = (high << i) | (low >> (64-i));
-  t->low  = low << i;
+  t->xbit = 0;
+  t->high = 0;
+  t->low  = i;
+
+  return *t;
+}
+
+int ipv6_addr_t::operator==(ip_v6word_t i)
+{
+  return (low == i);
+}
+
+ipv6_addr_t& ipv6_addr_t::operator<<(unsigned int i)
+{
+  // i is max 128 in fact, due to type size
+  // cyclic shift
+  ipv6_addr_t *t = new ipv6_addr_t(*this);
+  unsigned int c;
+
+  for (c = 1; c <= i; c++) {
+    t->xbit = 0 | (t->high >> 63);
+    t->high = (t->high << 1) | (t->low >> 63);
+    t->low  = t->low << 1;
+  }
 
   return *t;
 
@@ -1046,11 +1099,18 @@ ipv6_addr_t& ipv6_addr_t::operator<<(unsigned int i)
 
 ipv6_addr_t& ipv6_addr_t::operator>>(unsigned int i)
 {
-  ipv6_addr_t *t = new ipv6_addr_t(0,0,0);
-  
-  t->xbit = xbit >> i;
-  t->high  = (high >> i) | xbit << (64-i);
-  t->low = (low >> i) | (high << (64-i));
+  // i is max 128 in fact, due to type size
+  // cyclic shift
+  ipv6_addr_t *t = new ipv6_addr_t(*this);
+  unsigned int c;
+
+  for (c = 1; c <= i; c++) {
+    long long int p = t->xbit;
+    p = p << 63;
+    t->low  = (t->low >> 1) | (t->high << 63);
+    t->high = (t->high >> 1) | p;
+    t->xbit = 0;
+  }
   
   return *t;
 
@@ -1140,7 +1200,7 @@ void MPPrefixRanges::append_list(const MPPrefixRanges *src) {
 }
 
 // AND
-void MPPrefixRanges::and(MPPrefixRanges *src) {
+void MPPrefixRanges::_and(MPPrefixRanges *src) {
    MPPrefixRanges result = new MPPrefixRanges;
    insert_iterator<MPPrefixRanges> res_ins(result, result.begin());
    set_intersection(this->begin(), this->end(), src->begin(), src->end(), res_ins);
